--[[=============================================================================
    Basic Template for Receiver/Tuner Driver

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]------------
require "common.c4_driver_declarations"
require "common.c4_common"
require "common.c4_init"
require "common.c4_property"
require "common.c4_command"
require "common.c4_notify"
require "common.c4_network_connection"
require "common.c4_serial_connection"
require "common.c4_ir_connection"
require "common.c4_utils"
require "lib.c4_timer"
require "actions"
require "device_specific_commands"
require "device_messages"
require "receiver_init"
require "properties"
require "proxy_commands"
require "connections"
require "receiver.receiver_proxy_class"
require "receiver.receiver_proxy_commands"
require "receiver.receiver_proxy_notifies"


-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.driver = "2015.03.31"
end

--[[=============================================================================
    Constants
===============================================================================]]
RECEIVER_PROXY_BINDINGID = 5001
TUNER_PROXY_BINDINGID = 5002


--[[=============================================================================
    Initialization Code
===============================================================================]]
function ON_DRIVER_EARLY_INIT.main()
	
end

function ON_DRIVER_INIT.main()
     DRIVER_NAME = C4:GetDriverConfigInfo("name")
	SetLogName(DRIVER_NAME)

	-- TODO: If cloud based driver then uncomment the following line
	--ConnectURL()
end

function ON_DRIVER_LATEINIT.main()
	C4:urlSetTimeout(20)
end

function ON_DRIVER_EARLY_INIT.receiver_driver()

end

function ON_DRIVER_INIT.receiver_driver()
    -- TODO: Modify tVolumeRamping to have on entry per Output connection
    --index is mod 1000 value of output connection
    local tVolumeRamping = {
		[0] = {state = false,mode = "",},
		[1] = {state = false,mode = "",},
	}
	
    -- Create an instance of the ReceiverProxy class
    -- TODO: Change bProcessesDeviceMessages to false if Device Messages will not be processes
    local  bProcessesDeviceMessages = true
    gReceiverProxy = ReceiverProxy:new(RECEIVER_PROXY_BINDINGID, TUNER_PROXY_BINDINGID,  bProcessesDeviceMessages, tVolumeRamping)
    
    
    gNetworkStatusChangedTimer = c4_timer:new("NetworkStatusChangedTimer", 1000, "MILLISECONDS", OnNetworkStatusChangedTimerExpired)    
	
end

function ON_DRIVER_LATEINIT.receiver_driver()

end

--[[=============================================================================
    Driver Code
===============================================================================]]
function PackAndQueueCommand(...)
    local command_name = select(1, ...) or ""
    local command = select(2, ...) or ""
    local command_delay = select(3, ...) or tonumber(Properties["Command Delay Milliseconds"])
    local delay_units = select(4, ...) or "MILLISECONDS"
    LogTrace("PackAndQueueCommand(), command_name = " .. command_name .. ", command delay set to " .. command_delay .. " " .. delay_units)
    if (command == "") then
	   LogWarn("PackAndQueueCommand(), command_name = " .. command_name .. ", command string is empty - exiting PackAndQueueCommand()")
	   return
    end
	
	-- TODO: pack command with any any required starting or ending characters
    local cmd, stx, etx
    if (gControlMethod == "Network") then
		-- TODO: define any required starting or ending characters. 
		stx = "!"
		etx = "\r"
		cmd = stx .. command .. etx
    elseif (gControlMethod == "Serial") then
		-- TODO: define any required starting or ending characters. 
		stx = "!"
		etx = "\r"
		cmd = stx .. command .. etx
    elseif (gControlMethod == "IR") then
		cmd = command
    else
		LogWarn("PackAndQueueCommand(): gControlMethod is not valid, ".. gControlMethod)
		return
    end
    gCon:QueueCommand(cmd, command_delay, delay_units, command_name)	
	
end
